﻿using Kivii.Auth;
using Kivii.Finances.Entities;
using Kivii.Finances.Seeyon.Entities;
using Kivii.Seeyon.Entities;
using Kivii.Text;
using System;
using System.Collections.Generic;
using System.Diagnostics.Contracts;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;

namespace Kivii.Finances.Seeyon.Extensions
{
    internal static class ApplyExtension
    {
        public static string GetTemplateCodeSuffix(this string payeeName)
        {
            var rtns = string.Empty;
            if (payeeName.Contains("南京市产品质量监督检验院")) rtns = "ZJY";
            if (payeeName.Contains("深圳市宁深检验检测技术有限公司")) rtns = "NS";
            if (payeeName.Contains("江苏苏测智能装备检测有限公司")) rtns = "ZB";
            if (payeeName.Contains("江苏苏测检测认证有限公司")) rtns = "SC";
            return rtns;
        }

        public static string ConvertToChinese(this decimal Num)
        {
            string[] DX_SZ = { "零", "壹", "贰", "叁", "肆", "伍", "陆", "柒", "捌", "玖", "拾" };//大写数字  
            string[] DX_DW = { "元", "拾", "佰", "仟", "万", "拾", "佰", "仟", "亿", "拾", "佰", "仟", "万" };
            string[] DX_XSDS = { "角", "分" };//大些小数单位  
            if (Num == 0) return DX_SZ[0];

            Boolean IsXS_bool = false;//是否小数  

            string NumStr;//整个数字字符串  
            string NumStr_Zs;//整数部分  
            string NumSr_Xs = "";//小数部分  
            string NumStr_R = "";//返回的字符串  


            NumStr = Num.ToString();
            NumStr_Zs = NumStr;
            if (NumStr_Zs.Contains("."))
            {
                NumStr = Math.Round(Num, 2).ToString();
                NumStr_Zs = NumStr.Substring(0, NumStr.IndexOf("."));
                NumSr_Xs = NumStr.Substring((NumStr.IndexOf(".") + 1), (NumStr.Length - NumStr.IndexOf(".") - 1));
                IsXS_bool = true;
            }

            int k = 0;
            Boolean IsZeor = false;//整数中间连续0的情况  
            for (int i = 0; i < NumStr_Zs.Length; i++) //整数  
            {
                int j = int.Parse(NumStr_Zs.Substring(i, 1));
                if (j != 0)
                {
                    NumStr_R += DX_SZ[j] + DX_DW[NumStr_Zs.Length - i - 1];
                    IsZeor = false; //没有连续0  
                }
                else if (j == 0)
                {
                    k++;
                    if (!IsZeor && !(NumStr_Zs.Length == i + 1)) //等于0不是最后一位，连续0取一次  
                    {
                        //有问题  
                        if (NumStr_Zs.Length - i - 1 >= 4 && NumStr_Zs.Length - i - 1 <= 6)
                            NumStr_R += DX_DW[4] + "零";
                        else
                            if (NumStr_Zs.Length - i - 1 > 7)
                            NumStr_R += DX_DW[8] + "零";
                        else
                            NumStr_R += "零";

                        IsZeor = true;
                    }

                    if (NumStr_Zs.Length == i + 1)//  等于0且是最后一位 变成 XX元整  
                        NumStr_R += DX_DW[NumStr_Zs.Length - i - 1];
                }

            }
            if (NumStr_Zs.Length > 2 && k == NumStr_Zs.Length - 1)
                NumStr_R = NumStr_R.Remove(NumStr_R.IndexOf('零'), 1); //比如1000，10000元整的情况下 去0  

            if (!IsXS_bool) return NumStr_R + "整"; //如果没有小数就返回  
            else
            {
                for (int i = 0; i < NumSr_Xs.Length; i++)
                {
                    int j = int.Parse(NumSr_Xs.Substring(i, 1));
                    NumStr_R += DX_SZ[j] + DX_XSDS[NumSr_Xs.Length - i - 1];
                }
            }

            return NumStr_R;
        }

        public static ResponseToken RequestRestToken(this JsonServiceClient client, string loginName)
        {
            if (client == null) throw new ArgumentNullException("client");
            if (loginName.IsNullOrEmpty()) throw new ArgumentNullException("loginName");

            try
            {
                var request = new RequestRestToken();
                request.userName = Configs.restUserName;
                request.password = Configs.restPassword;
                request.loginName = loginName;
                Console.WriteLine("---------------------------------------------------------");
                Console.WriteLine(loginName);
                Console.WriteLine("---------------------------------------------------------");
                var rtns = client.Post<ResponseToken>($"{Configs.RouteRestToken}", request);
                return rtns;
            }
            catch (Exception ex)
            {
                throw new Exception($"{ex.Message}，可能原因：当前登录人{loginName}，无法与OA账号对应，请联系管理员确认！");
            }
        }

        public static ResponseOrgMember RequestOrgMember(this JsonServiceClient client, string loginName, ResponseToken token)
        {
            if (client == null) throw new ArgumentNullException("client");
            if (loginName.IsNullOrEmpty()) throw new ArgumentNullException("loginName");

            try
            {
                var rtns = client.Get<ResponseOrgMember>($"{Configs.RouteOrgMember}?token={token.id}&&loginName={loginName}");
                return rtns;
            }
            catch (Exception ex)
            {
                throw new Exception($"{ex.Message}，可能原因：当前登录人{loginName}，无法与OA账号对应，请联系管理员确认！");
            }
        }


        #region 账单申请发票

        public static ResponseForm<ResponseData> RequestBillForm0710(this JsonServiceClient client, FormMain_OrdinaryInvoice0710 form, ResponseToken token)
        {
            if (client == null) throw new ArgumentNullException("client");
            if (form == null) throw new ArgumentNullException("form");
            //var token = client.RequestRestToken(session.FullName);
            var request = new RequestForm<RequestForm_Apply>();
            request.appName = Configs.appName;//"collaboration";
            request.data = new RequestData<RequestForm_Apply>();
            request.data.templateCode = Configs.templateCodeOrdinaryInvoice;//"JYHTSQ";
            var templateCodeSuffix = form.收款单位名称.GetTemplateCodeSuffix();
            if (!templateCodeSuffix.IsNullOrEmpty()) request.data.templateCode = $"{Configs.templateCodeOrdinaryInvoice}_{templateCodeSuffix}";
            request.data.draft = "0";
            //request.data.senderLoginName = token.bindingUser.loginName;
            request.data.subject = $"到账发票申请({token.bindingUser.name} {DateTime.Now:yyyy-MM-dd HH:mm})";
            request.data.data = new RequestForm_Apply();
            request.data.data.formmain_0710 = new FormMain_OrdinaryInvoice0710();
            request.data.data.formmain_0710 = form;
            Console.WriteLine("-------------------------------------------------------------------------");
            Console.WriteLine(JsonSerializer.SerializeToString(request));
            Console.WriteLine("-------------------------------------------------------------------------");
            var rtns = client.Post<ResponseForm<ResponseData>>($"{Configs.RouteRequestForm}?token={token.id}", request);
            return rtns;
        }

        public static ResponseForm<ResponseData> RequestBillForm0318(this JsonServiceClient client, FormMain_AdvanceInvoice0318 form, ResponseToken token)
        {
            if (client == null) throw new ArgumentNullException("client");
            if (form == null) throw new ArgumentNullException("form");
            //var token = client.RequestRestToken(session.FullName);
            var request = new RequestForm<RequestForm_Apply>();
            request.appName = Configs.appName;//"collaboration";
            request.data = new RequestData<RequestForm_Apply>();
            request.data.templateCode = Configs.templateCodeAdvanceInvoice;//"JYHTSQ";
            var templateCodeSuffix = form.收款单位.GetTemplateCodeSuffix();
            if (!templateCodeSuffix.IsNullOrEmpty()) request.data.templateCode = $"{Configs.templateCodeAdvanceInvoice}_{templateCodeSuffix}";
            request.data.draft = "0";
            //request.data.senderLoginName = token.bindingUser.loginName;
            request.data.subject = $"预借发票申请({token.bindingUser.name} {DateTime.Now:yyyy-MM-dd HH:mm})";
            request.data.data = new RequestForm_Apply();
            request.data.data.formmain_0318 = new FormMain_AdvanceInvoice0318();
            request.data.data.formmain_0318 = form;
            Console.WriteLine("-------------------------------------------------------------------------");
            Console.WriteLine(JsonSerializer.SerializeToString(request));
            Console.WriteLine("-------------------------------------------------------------------------");
            var rtns = client.Post<ResponseForm<ResponseData>>($"{Configs.RouteRequestForm}?token={token.id}", request);
            return rtns;
        }

        public static FormMain_OrdinaryInvoice0710 PayedConvertToForm(this List<InvoiceApply> applys, ResponseOrgMember orgMember)
        {
            //var orgMember = client.RequestOrgMember(session.FullName);
            if (orgMember == null) throw new Exception("未找到OA登录人员信息！");
            var apply = applys[0];
            var bizKvids = applys.ConvertAll(o => o.Kvid.ToString());
            //var strBizKvids = JsonSerializer.SerializeToString(bizKvids);
            var amount = applys.Sum(o => o.Amount);
            var rtns = new FormMain_OrdinaryInvoice0710();
            //var settlementType = "code|MonthlySettlement";//默认月结
            rtns.业务类型 = typeof(Entities.BillApply).FullName;
            rtns.业务编号 = bizKvids;
            //rtns.单据编号 = apply.SerialNumber;
            rtns.申请日期 = DateTime.Now.ToString("yyyy-MM-dd");
            rtns.申请部门 = orgMember.orgDepartmentId;//KiviiContext.CurrentMember.DepartmentName;
            rtns.申请人 = orgMember.id;//apply.OperatorName;
            rtns.单位 = orgMember.orgAccountId;
            rtns.发票类型 = apply.Type == "增值税专用发票" ? "-4189929216282050282" : (apply.Type == "增值税普通发票" ? "4529935671729733794" : "4728848026009816823");
            rtns.数量 = "1";
            rtns.总金额 = amount.ToString();
            rtns.金额大写 = amount.ConvertToChinese();
            rtns.付款单位名称 = apply.PayerName;
            rtns.纳税人识别号 = apply.PayerTaxNumber;
            rtns.地址电话 = $"{apply.PayerCompanyAddress}{apply.PayerPhone}";
            rtns.开户行及账号 = $"{apply.PayerRegisteredBank}{apply.PayerBankAccount}";
            rtns.收款单位名称 = apply.PayeeName;
            rtns.收款单位纳税人识别号 = apply.PayeeTaxNumber;
            rtns.收款单位地址电话 = $"{apply.PayeeCompanyAddress}{apply.PayeePhone}";
            rtns.收款单位开户行及账号 = $"{apply.PayeeRegisteredBank}{apply.PayeeBankAccount}";
            rtns.申请原因 = apply.Remark;
            rtns.收款单位名称 = apply.PayeeName;
            return rtns;
        }

        public static FormMain_OrdinaryInvoice0710 PayedConvertToForm(this InvoiceApply apply, ResponseOrgMember orgMember)
        {
            //var orgMember = client.RequestOrgMember(session.FullName);
            if (orgMember == null) throw new Exception("未找到OA登录人员信息！");
            var bizKvids = new List<string>();
            bizKvids.Add(apply.Kvid.ToString());
            var rtns = new FormMain_OrdinaryInvoice0710();
            //var settlementType = "code|MonthlySettlement";//默认月结
            rtns.业务类型 = typeof(Entities.BillApply).FullName;
            rtns.业务编号 = bizKvids;
            //rtns.单据编号 = apply.SerialNumber;
            rtns.申请日期 = DateTime.Now.ToString("yyyy-MM-dd");
            rtns.申请部门 = orgMember.orgDepartmentId;//KiviiContext.CurrentMember.DepartmentName;
            rtns.申请人 = orgMember.id;//apply.OperatorName;
            rtns.单位 = orgMember.orgAccountId;
            rtns.发票类型 = apply.Type == "增值税专用发票" ? "-4189929216282050282" : (apply.Type == "增值税普通发票" ? "4529935671729733794" : "4728848026009816823");
            rtns.数量 = "1";
            rtns.总金额 = apply.Amount.ToString();
            rtns.金额大写 = apply.Amount.ConvertToChinese();
            rtns.付款单位名称 = apply.PayerName;
            rtns.纳税人识别号 = apply.PayerTaxNumber;
            rtns.地址电话 = $"{apply.PayerCompanyAddress}{apply.PayerPhone}";
            rtns.开户行及账号 = $"{apply.PayerRegisteredBank}{apply.PayerBankAccount}";
            rtns.收款单位名称 = apply.PayeeName;
            rtns.收款单位纳税人识别号 = apply.PayeeTaxNumber;
            rtns.收款单位地址电话 = $"{apply.PayeeCompanyAddress}{apply.PayeePhone}";
            rtns.收款单位开户行及账号 = $"{apply.PayeeRegisteredBank}{apply.PayeeBankAccount}";
            rtns.申请原因 = apply.Remark;
            rtns.收款单位名称 = apply.PayeeName;
            return rtns;
        }

        public static FormMain_AdvanceInvoice0318 DebitConvertToForm(this List<InvoiceApply> applys, ResponseOrgMember orgMember)
        {
            //var orgMember = client.RequestOrgMember(session.FullName);
            if (orgMember == null) throw new Exception("未找到OA登录人员信息！");
            var apply = applys[0];
            var bizKvids = applys.ConvertAll(o => o.Kvid.ToString());
            //var strBizKvids = JsonSerializer.SerializeToString(bizKvids);
            var amount = applys.Sum(o => o.Amount);
            var rtns = new FormMain_AdvanceInvoice0318();
            //var settlementType = "code|MonthlySettlement";//默认月结
            rtns.业务类型 = typeof(Entities.BillApply).FullName;
            rtns.业务编号 = bizKvids;
            //rtns.单据编号 = apply.SerialNumber;
            rtns.申请日期 = DateTime.Now.ToString("yyyy-MM-dd");
            rtns.申请部门 = orgMember.orgDepartmentId;//KiviiContext.CurrentMember.DepartmentName;
            rtns.申请人 = orgMember.id;//apply.OperatorName;
            rtns.单位 = orgMember.orgAccountId;
            rtns.发票类型 = apply.Type == "增值税专用发票" ? "-4189929216282050282" : (apply.Type == "增值税普通发票" ? "4529935671729733794" : "4728848026009816823");
            rtns.数量 = "1";
            rtns.总金额 = amount.ToString();
            rtns.金额大写 = amount.ConvertToChinese();
            rtns.付款单位名称 = apply.PayerName;
            rtns.纳税人识别号 = apply.PayerTaxNumber;
            rtns.地址电话 = $"{apply.PayerCompanyAddress}{apply.PayerPhone}";
            rtns.开户行及账号 = $"{apply.PayerRegisteredBank}{apply.PayerBankAccount}";
            rtns.收款单位 = apply.PayeeName;
            rtns.收款单位纳税人识别号 = apply.PayeeTaxNumber;
            rtns.收款单位地址电话 = $"{apply.PayeeCompanyAddress}{apply.PayeePhone}";
            rtns.收款单位开户行及账号 = $"{apply.PayeeRegisteredBank}{apply.PayeeBankAccount}";
            rtns.申请原因 = apply.Remark;
            rtns.收款单位 = apply.PayeeName;
            return rtns;
        }

        public static FormMain_AdvanceInvoice0318 DebitConvertToForm(this InvoiceApply apply, ResponseOrgMember orgMember)
        {
            //var orgMember = client.RequestOrgMember(session.FullName);
            if (orgMember == null) throw new Exception("未找到OA登录人员信息！");
            var bizKvids = new List<string>();
            bizKvids.Add(apply.Kvid.ToString());
            var rtns = new FormMain_AdvanceInvoice0318();
            //var settlementType = "code|MonthlySettlement";//默认月结
            rtns.业务类型 = typeof(Entities.BillApply).FullName;
            rtns.业务编号 = bizKvids;
            //rtns.单据编号 = apply.SerialNumber;
            rtns.申请日期 = DateTime.Now.ToString("yyyy-MM-dd");
            rtns.申请部门 = orgMember.orgDepartmentId;//KiviiContext.CurrentMember.DepartmentName;
            rtns.申请人 = orgMember.id;//apply.OperatorName;
            rtns.单位 = orgMember.orgAccountId;
            rtns.发票类型 = apply.Type == "增值税专用发票" ? "-4189929216282050282" : (apply.Type == "增值税普通发票" ? "4529935671729733794" : "4728848026009816823");
            rtns.数量 = "1";
            rtns.总金额 = apply.Amount.ToString();
            rtns.金额大写 = apply.Amount.ConvertToChinese();
            rtns.付款单位名称 = apply.PayerName;
            rtns.纳税人识别号 = apply.PayerTaxNumber;
            rtns.地址电话 = $"{apply.PayerCompanyAddress}{apply.PayerPhone}";
            rtns.开户行及账号 = $"{apply.PayerRegisteredBank}{apply.PayerBankAccount}";
            rtns.收款单位 = apply.PayeeName;
            rtns.收款单位纳税人识别号 = apply.PayeeTaxNumber;
            rtns.收款单位地址电话 = $"{apply.PayeeCompanyAddress}{apply.PayeePhone}";
            rtns.收款单位开户行及账号 = $"{apply.PayeeRegisteredBank}{apply.PayeeBankAccount}";
            rtns.申请原因 = apply.Remark;
            rtns.收款单位 = apply.PayeeName;
            return rtns;
        }
        #endregion

        #region 发票作废
        public static ResponseForm<ResponseData> RequestOffsetForm0712(this JsonServiceClient client, FormMain_InvoiceVoid0712 form, ResponseToken token)
        {
            if (client == null) throw new ArgumentNullException("client");
            if (form == null) throw new ArgumentNullException("form");
            //var token = client.RequestRestToken(session.FullName);
            var request = new RequestForm<RequestForm_Apply>();
            request.appName = Configs.appName;//"collaboration";
            request.data = new RequestData<RequestForm_Apply>();
            request.data.templateCode = Configs.templateCodeInvoiceVoid;//"JYHTSQ";
            var templateCodeSuffix = form.收款单位名称.GetTemplateCodeSuffix();
            if (!templateCodeSuffix.IsNullOrEmpty()) request.data.templateCode = $"{Configs.templateCodeInvoiceVoid}_{templateCodeSuffix}";
            request.data.draft = "0";
            //request.data.senderLoginName = token.bindingUser.loginName;
            request.data.subject = $"发票当月作废申请({token.bindingUser.name} {DateTime.Now:yyyy-MM-dd HH:mm})";
            request.data.data = new RequestForm_Apply();
            request.data.data.formmain_0712 = new FormMain_InvoiceVoid0712();
            request.data.data.formmain_0712 = form;
            Console.WriteLine("-------------------------------------------------------------------------");
            Console.WriteLine(JsonSerializer.SerializeToString(request));
            Console.WriteLine("-------------------------------------------------------------------------");
            var rtns = client.Post<ResponseForm<ResponseData>>($"{Configs.RouteRequestForm}?token={token.id}", request);
            return rtns;
        }

        public static FormMain_InvoiceVoid0712 OffsetConvertToForm(this Invoice invoice, ResponseOrgMember orgMember)
        {
            //var orgMember = client.RequestOrgMember(session.FullName);
            if (orgMember == null) throw new Exception("未找到OA登录人员信息！");
            var bizKvids = new List<string>();
            bizKvids.Add(invoice.Kvid.ToString());
            var rtns = new FormMain_InvoiceVoid0712();
            //var settlementType = "code|MonthlySettlement";//默认月结
            rtns.业务类型 = typeof(InvoiceOffsetApply).FullName;
            rtns.业务编号 = bizKvids;
            //rtns.单据编号 = invoice.SerialNumber;
            rtns.申请日期 = DateTime.Now.ToString("yyyy-MM-dd");
            rtns.申请部门 = orgMember.orgDepartmentId;//KiviiContext.CurrentMember.DepartmentName;
            rtns.申请人 = orgMember.id;//apply.OperatorName;
            rtns.原发票号 = invoice.SerialNumber;
            rtns.原发票类型 = invoice.Type == "增值税专用发票" ? "-4189929216282050282" : (invoice.Type == "增值税普通发票" ? "4529935671729733794" : "4728848026009816823");
            rtns.原发票开票单位 = invoice.PayerName;
            rtns.收款单位名称 = invoice.PayeeName;
            rtns.原开票金额 = invoice.Amount.ToString();
            return rtns;
        }


        public static ResponseForm<ResponseData> RequestFlushForm0338(this JsonServiceClient client, FormMain_InvoiceRedFlush0338 form, ResponseToken token)
        {
            if (client == null) throw new ArgumentNullException("client");
            if (form == null) throw new ArgumentNullException("form");
            //var token = client.RequestRestToken(session.FullName);
            var request = new RequestForm<RequestForm_Apply>();
            request.appName = Configs.appName;//"collaboration";
            request.data = new RequestData<RequestForm_Apply>();
            request.data.templateCode = Configs.templateCodeInvoiceRedOffset;//"JYHTSQ";
            var templateCodeSuffix = form.收款单位名称.GetTemplateCodeSuffix();
            if (!templateCodeSuffix.IsNullOrEmpty()) request.data.templateCode = $"{Configs.templateCodeInvoiceRedOffset}_{templateCodeSuffix}";
            request.data.draft = "0";
            //request.data.senderLoginName = token.bindingUser.loginName;
            request.data.subject = $"跨月发票红冲申请({token.bindingUser.name} {DateTime.Now:yyyy-MM-dd HH:mm})";
            request.data.data = new RequestForm_Apply();
            request.data.data.formmain_0338 = new FormMain_InvoiceRedFlush0338();
            request.data.data.formmain_0338 = form;
            Console.WriteLine("-------------------------------------------------------------------------");
            Console.WriteLine(JsonSerializer.SerializeToString(request));
            Console.WriteLine("-------------------------------------------------------------------------");
            var rtns = client.Post<ResponseForm<ResponseData>>($"{Configs.RouteRequestForm}?token={token.id}", request);
            return rtns;
        }

        public static FormMain_InvoiceRedFlush0338 FlushConvertToForm(this Invoice invoice, ResponseOrgMember orgMember)
        {
            //var orgMember = client.RequestOrgMember(session.FullName);
            if (orgMember == null) throw new Exception("未找到OA登录人员信息！");
            var bizKvids = new List<string>();
            bizKvids.Add(invoice.Kvid.ToString());
            var rtns = new FormMain_InvoiceRedFlush0338();
            //var settlementType = "code|MonthlySettlement";//默认月结
            rtns.业务类型 = typeof(InvoiceRedFlushApply).FullName;
            rtns.业务编号 = bizKvids;
            //rtns.单据编号 = invoice.SerialNumber;
            rtns.申请日期 = DateTime.Now.ToString("yyyy-MM-dd");
            rtns.申请部门 = orgMember.orgDepartmentId;//KiviiContext.CurrentMember.DepartmentName;
            rtns.申请人 = orgMember.id;//apply.OperatorName;
            rtns.是否为预借发票 = invoice.Category == "Debit" ? "-9179045864512534208" : "-7622688603542042575";
            rtns.原发票号 = invoice.SerialNumber;
            rtns.原发票类型 = invoice.Type == "增值税专用发票" ? "-4189929216282050282" : (invoice.Type == "增值税普通发票" ? "4529935671729733794" : "4728848026009816823");
            rtns.原发票开票单位 = invoice.PayerName;
            rtns.收款单位名称 = invoice.PayeeName;
            rtns.原开票金额 = invoice.Amount.ToString();
            return rtns;
        }

        #endregion

        #region 推送部门收入相关
        public static string RequestGetFormXml(this JsonServiceClient client, ResponseToken token, DateTime? beginTime = null, DateTime? endTime = null)
        {

            if (client == null) throw new ArgumentNullException("client");

            if (beginTime == null) beginTime = DateTime.Now;
            if (endTime == null) endTime = DateTime.Now;
            try
            {
                var rtns = client.Get<string>($"{Configs.RouteFormExport}{Configs.templateCodeIncome}?token={token.id}&beginDateTime={beginTime:yyyy-MM-dd}&endDateTime={endTime:yyyy-MM-dd}");
                return rtns;
            }
            catch (Exception ex)
            {

                throw ex;
            }
        }

        public static string RequestUploadFormXml(this JsonServiceClient client, List<OaDepartIncome> list, string loginName, ResponseToken token)
        {

            if (client == null) throw new ArgumentNullException("client");

            try
            {
                var rtns=string.Empty;
                foreach (var item in list)
                {
                    var data = GetSendXml(item);
                    var body = new Dictionary<string, string>();
                    body.Add("templateCode", Configs.templateCodeIncome);
                    body.Add("loginName", loginName);
                    body.Add("dataXml", data);
                    Console.WriteLine("---------------------------------------------------------");
                    Console.WriteLine($"{Configs.RouteFormImport}{Configs.templateCodeIncome}?token={token.id}");
                    Console.WriteLine("---------------------------------------------------------");
                    Console.WriteLine(JsonSerializer.SerializeToString(body));
                    Console.WriteLine("---------------------------------------------------------");

                    var resp = client.Post<string>($"{Configs.RouteFormImport}{Configs.templateCodeIncome}?token={token.id}", body);
                    rtns = resp;
                    Thread.Sleep(200);
                }

                return rtns;
            }
            catch (Exception ex)
            {

                throw ex;
            }
        }


        private static string GetSendXml(OaDepartIncome item)
        {
            StringBuilder sb = new StringBuilder();

            //sb.Append("<forms  version=\"2.1\"><formExport><summary  id=\"-8487669074587464173\" name=\"formmain_0928\"/><definitions><column  id=\"field0005\" type=\"4\" name=\"年份\" isNullable=\"false\" length=\"20\"/><column  id=\"field0006\" type=\"0\" name=\"部门\" isNullable=\"false\" length=\"20\"/><column  id=\"field0007\" type=\"4\" name=\"核算业务总收入\" isNullable=\"false\" length=\"20\"/><column  id=\"field0008\" type=\"4\" name=\"年度业务目标\" isNullable=\"false\" length=\"20\"/><column  id=\"field0009\" type=\"4\" name=\"完成率\" isNullable=\"false\" length=\"20\"/><column  id=\"field0050\" type=\"3\" name=\"日期\" isNullable=\"false\" length=\"255\"/></definitions><values>");
            sb.Append("<forms version=\"2.1\"><formExport><summary id=\"-8487669074587464173\" name=\"formmain_0928\"/><definitions><column id=\"field0005\" type=\"4\" name=\"年份\" isNullable=\"false\" length=\"20\"/><column id=\"field0006\" type=\"0\" name=\"部门\" isNullable=\"false\" length=\"20\"/><column id=\"field0007\" type=\"4\" name=\"核算业务总收入\" isNullable=\"false\" length=\"20\"/><column id=\"field0008\" type=\"4\" name=\"年度业务目标\" isNullable=\"false\" length=\"20\"/><column id=\"field0009\" type=\"4\" name=\"完成率\" isNullable=\"false\" length=\"20\"/><column id=\"field0050\" type=\"3\" name=\"日期\" isNullable=\"false\" length=\"255\"/><column id=\"field0057\" type=\"0\" name=\"部门简称\" isNullable=\"false\" length=\"255\"/><column id=\"field0058\" type=\"0\" name=\"部门排序\" isNullable=\"false\" length=\"255\"/><column id=\"field0059\" type=\"4\" name=\"待认领流水条数\" isNullable=\"false\" length=\"20\"/><column id=\"field0060\" type=\"4\" name=\"待认领流水金额\" isNullable=\"false\" length=\"20\"/></definitions><values>");


            //sb.Append($"<column name=\"年份\"><value><![CDATA[{item.Year}]]></value></column><column name=\"部门\"><value><![CDATA[{item.DepartId}]]></value></column><column name=\"核算业务总收入\"><value><![CDATA[{item.AmountTotal}]]></value></column><column name=\"年度业务目标\"><value><![CDATA[{item.AmountTarget}]]></value></column><column name=\"完成率\"><value><![CDATA[{item.Rate}]]></value></column><column name=\"日期\"><value><![CDATA[{item.OperateTime:yyyy-MM-dd}]]></value></column>");
            sb.Append($"<column name=\"年份\"><value><![CDATA[{item.Year}]]></value></column><column name=\"部门\"><value><![CDATA[{item.DepartId}]]></value></column><column name=\"核算业务总收入\"><value><![CDATA[{item.AmountTotal}]]></value></column><column name=\"年度业务目标\"><value><![CDATA[{item.AmountTarget}]]></value></column><column name=\"完成率\"><value><![CDATA[{item.Rate}]]></value></column><column name=\"日期\"><value><![CDATA[{item.OperateTime:yyyy-MM-dd}]]></value></column><column name=\"部门简称\"><value><![CDATA[{item.Depart}]]></value></column><column name=\"部门排序\"><value><![CDATA[0]]></value></column><column name=\"待认领流水条数\"><value>{item.QuantityUnSplit}</value></column><column name=\"待认领流水金额\"><value>{item.AmountUnSplit}</value></column>");

            sb.Append("</values><subForms><subForm><definitions><column  id=\"field0051\" type=\"4\" name=\"序号\" isNullable=\"false\" length=\"20\"/><column  id=\"field0056\" type=\"4\" name=\"收入类型\" isNullable=\"false\" length=\"20\"/><column  id=\"field0052\" type=\"4\" name=\"认领\" isNullable=\"false\" length=\"20\"/><column  id=\"field0053\" type=\"4\" name=\"划入\" isNullable=\"false\" length=\"20\"/><column  id=\"field0054\" type=\"4\" name=\"划出\" isNullable=\"false\" length=\"20\"/><column  id=\"field0055\" type=\"4\" name=\"小计\" isNullable=\"false\" length=\"20\"/><column  id=\"field0051\" type=\"4\" name=\"序号\" isNullable=\"false\" length=\"20\"/><column  id=\"field0056\" type=\"4\" name=\"收入类型\" isNullable=\"false\" length=\"20\"/><column  id=\"field0052\" type=\"4\" name=\"认领\" isNullable=\"false\" length=\"20\"/><column  id=\"field0053\" type=\"4\" name=\"划入\" isNullable=\"false\" length=\"20\"/><column  id=\"field0054\" type=\"4\" name=\"划出\" isNullable=\"false\" length=\"20\"/><column  id=\"field0055\" type=\"4\" name=\"小计\" isNullable=\"false\" length=\"20\"/><column  id=\"field0051\" type=\"4\" name=\"序号\" isNullable=\"false\" length=\"20\"/><column  id=\"field0056\" type=\"4\" name=\"收入类型\" isNullable=\"false\" length=\"20\"/><column  id=\"field0052\" type=\"4\" name=\"认领\" isNullable=\"false\" length=\"20\"/><column  id=\"field0053\" type=\"4\" name=\"划入\" isNullable=\"false\" length=\"20\"/><column  id=\"field0054\" type=\"4\" name=\"划出\" isNullable=\"false\" length=\"20\"/><column  id=\"field0055\" type=\"4\" name=\"小计\" isNullable=\"false\" length=\"20\"/><column  id=\"field0051\" type=\"4\" name=\"序号\" isNullable=\"false\" length=\"20\"/><column  id=\"field0056\" type=\"4\" name=\"收入类型\" isNullable=\"false\" length=\"20\"/><column  id=\"field0052\" type=\"4\" name=\"认领\" isNullable=\"false\" length=\"20\"/><column  id=\"field0053\" type=\"4\" name=\"划入\" isNullable=\"false\" length=\"20\"/><column  id=\"field0054\" type=\"4\" name=\"划出\" isNullable=\"false\" length=\"20\"/><column  id=\"field0055\" type=\"4\" name=\"小计\" isNullable=\"false\" length=\"20\"/><column  id=\"field0051\" type=\"4\" name=\"序号\" isNullable=\"false\" length=\"20\"/><column  id=\"field0056\" type=\"4\" name=\"收入类型\" isNullable=\"false\" length=\"20\"/><column  id=\"field0052\" type=\"4\" name=\"认领\" isNullable=\"false\" length=\"20\"/><column  id=\"field0053\" type=\"4\" name=\"划入\" isNullable=\"false\" length=\"20\"/><column  id=\"field0054\" type=\"4\" name=\"划出\" isNullable=\"false\" length=\"20\"/><column  id=\"field0055\" type=\"4\" name=\"小计\" isNullable=\"false\" length=\"20\"/><column  id=\"field0051\" type=\"4\" name=\"序号\" isNullable=\"false\" length=\"20\"/><column  id=\"field0056\" type=\"4\" name=\"收入类型\" isNullable=\"false\" length=\"20\"/><column  id=\"field0052\" type=\"4\" name=\"认领\" isNullable=\"false\" length=\"20\"/><column  id=\"field0053\" type=\"4\" name=\"划入\" isNullable=\"false\" length=\"20\"/><column  id=\"field0054\" type=\"4\" name=\"划出\" isNullable=\"false\" length=\"20\"/><column  id=\"field0055\" type=\"4\" name=\"小计\" isNullable=\"false\" length=\"20\"/></definitions><values>");

            foreach (var detail in item.Details)
            {
                //sb.Append("<row><column  name=\"序号\"><value><![CDATA[1]]></value></column><column  name=\"收入类型\"><value><![CDATA[0]]></value></column><column  name=\"认领\"><value><![CDATA[13.00]]></value></column><column  name=\"划入\"><value><![CDATA[13.00]]></value></column><column  name=\"划出\"><value><![CDATA[13.00]]></value></column><column  name=\"小计\"><value><![CDATA[13.00]]></value></column></row>");

                sb.Append($"<row><column name=\"序号\"><value><![CDATA[{item.Details.IndexOf(detail)}]]></value></column><column name=\"收入类型\"><value><![CDATA[{detail.TypeEx}]]></value></column><column name=\"认领\"><value><![CDATA[{detail.AmountClaim}]]></value></column><column name=\"划入\"><value><![CDATA[{detail.AmountIn}]]></value></column><column name=\"划出\"><value><![CDATA[{detail.AmountOut}]]></value></column><column name=\"小计\"><value><![CDATA[{detail.AmountTotal}]]></value></column></row>");
            }

            sb.Append("</values></subForm></subForms></formExport></forms>");
            return sb.ToString();
        }
        #endregion

        #region 获取OA合同数据
        public static ResponseForm<OaContract> RequestGetOaContract(this JsonServiceClient client, RequestOaContract item, ResponseToken token)
        {

            if (client == null) throw new ArgumentNullException("client");

            try
            {
                var rtns = client.Post<ResponseForm<OaContract>>($"{Configs.RouteCap4FormImport}?token={token.id}", item);
                return rtns;
            }
            catch (Exception ex)
            {

                throw ex;
            }
        }
        #endregion
    }
}
