﻿using Kivii.Finances.Seeyon.Entities;
using Kivii.Finances.Seeyon.Extensions;
using Kivii.Web;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Kivii.Finances.Seeyon
{
    public class OaDepartIncomeUpload : RestfulExecution<OaDepartIncome>
    {
        public List<OaDepartIncome> Items { get; set; }

        public string loginName { get; set; }

        public override object OnExecution(IRequest req, IResponse res)
        {
            var client = new JsonServiceClient(Configs.BaseUrl);
            var token = client.RequestRestToken(loginName);
            var reps = client.RequestUploadFormXml(Items, loginName, token);
            var rtns = new RestfulExecutionResponse<string>();
            rtns.Results = new List<string>();

            rtns.Results.Add(reps);
            return rtns;
        }
    }

    public class OaDepartIncomeGetFormXml : RestfulExecution<OaDepartIncome>
    {

        public string loginName { get; set; }
        public DateTime? BeginTime { get; set; }
        public DateTime? EndTime { get; set; }

        public override object OnExecution(IRequest req, IResponse res)
        {
            var client = new JsonServiceClient(Configs.BaseUrl);
            var token = client.RequestRestToken(loginName);
            var reps = client.RequestGetFormXml(token, BeginTime, EndTime);
            var rtns = new HttpResult
            {
                Response = reps
            };
            //var rtns = new RestfulExecutionResponse<string>();
            //rtns.Results = new List<string>();

            //rtns.Results.Add(reps);
            return rtns;
        }
    }

    [RequiresAnyRole(SystemRoles.Everyone)]
    public class ContractGetFromOa : RestfulExecution<Contract>
    {
        /// <summary>
        /// 导出数据的起始时间.eg:2020-06-30
        /// </summary>
        public DateTime BeginTime { get; set; }

        /// <summary>
        /// 导出数据的截止创建时间.eg:2020-06-30
        /// </summary>
        public DateTime EndTime { get; set; }

        /// <summary>
        /// 权限id
        /// </summary>
        public string rightId { get; set; } = Configs.templateCodeContractRightId;


        public string loginName { get; set; }

        public string Filter { get; set; }

        public override object OnExecution(IRequest req, IResponse res)
        {
            if (loginName.IsNullOrEmpty())
            {
                var session = KiviiContext.Request.GetSession();
                loginName = session.FullName;
            }
            var item = new RequestOaContract();
            if (BeginTime == DateTime.MinValue) item.beginDateTime = $"{DateTime.Now.Year}-01-01";
            else item.beginDateTime = $"{BeginTime.ToString("yyyy-MM-dd")}";
            if (EndTime == DateTime.MinValue) item.endDateTime = DateTime.Now.ToString("yyyy-MM-dd");
            else item.endDateTime = $"{EndTime.ToString("yyyy-MM-dd")}";
            var client = new JsonServiceClient(Configs.BaseUrl);
            var token = client.RequestRestToken(loginName);
            var reps = client.RequestGetOaContract(item, token);
            var rtns = new RestfulQueryResponse<Contract>();
            rtns.Results = new List<Contract>();

            foreach (var data in reps.data.data.data)
            {
                var oaData = data.masterData;
                var rtn = new Contract();
                rtn.ContractNumber = oaData.Field0063.showValue;
                rtn.SerialNumber = oaData.Field0081.showValue;
                rtn.Year = oaData.Field0080.showValue;
                rtn.Field0060 = oaData.Field0060.showValue;
                rtn.Field0062 = oaData.Field0062.showValue;
                rtn.ApplierName = oaData.Field0059.showValue;
                //if (DateTime.TryParse(oaData.Field0043.showValue, out var field0043))
                //{
                //    rtn.ApplyTime = field0043;
                //}
                rtn.ApplyTime = oaData.Field0043.showValue;
                rtn.ApplyDepart = oaData.Field0078.showValue;
                rtn.DepartCode = oaData.Field0079.showValue;
                rtn.DepartmentName = oaData.Field0096.showValue;
                rtn.DepartmentCode = oaData.Field0097.showValue;
                rtn.Field0084 = oaData.Field0084.showValue;
                rtn.Field0083 = oaData.Field0083.showValue;
                rtn.Name = oaData.Field0001.showValue;
                rtn.Field0058 = oaData.Field0058.showValue;
                rtn.DepartmentTest = oaData.Field0002.showValue;
                rtn.LeaderName = oaData.Field0003.showValue;
                rtn.CustomerName = oaData.Field0004.showValue;
                rtn.CustomerAddress = oaData.Field0005.showValue;
                rtn.CustomerLeaderName = oaData.Field0006.showValue;
                rtn.CustomerTax = oaData.Field0007.showValue;
                rtn.CustomerContactName = oaData.Field0008.showValue;
                rtn.CustomerContactNumber = oaData.Field0009.showValue;
                rtn.PayerName = oaData.Field0048.showValue;
                rtn.Type = oaData.Field0011.showValue;
                rtn.Field0072 = oaData.Field0072.showValue;
                rtn.AgreeYear = oaData.Field0012.showValue;
                //if (DateTime.TryParse(oaData.Field0045.showValue, out var field0045))
                //{
                //    rtn.AgreeBeginTime = field0045;
                //}
                //if (DateTime.TryParse(oaData.Field0046.showValue, out var field0046))
                //{
                //    rtn.AgreeEndTime = field0046;
                //}
                rtn.AgreeBeginTime = oaData.Field0045.showValue;
                rtn.AgreeEndTime = oaData.Field0046.showValue;
                rtn.RateText = oaData.Field0014.showValue;
                rtn.Rate = oaData.Field0019.showValue;
                rtn.AgreeType = oaData.Field0015.showValue;
                rtn.Remark = oaData.Field0017.showValue;
                rtn.Amount = oaData.Field0020.showValue;
                rtn.PayType = oaData.Field0022.showValue;
                rtn.AmountFirstPay = oaData.Field0023.showValue;
                rtn.Quantity = oaData.Field0033.showValue;
                rtn.LimsNumber = oaData.Field0085.showValue;
                rtn.Field0064 = oaData.Field0064.showValue;
                //if (DateTime.TryParse(oaData.Field0070.showValue, out var field0070))
                //{
                //    rtn.FirstPayTime = field0070;
                //}
                //if (DateTime.TryParse(oaData.Field0089.showValue, out var field0089))
                //{
                //    rtn.StampTime = field0089;
                //}
                rtn.FirstPayTime = oaData.Field0070.showValue;
                rtn.StampTime = oaData.Field0089.showValue;
                rtn.Field0074 = oaData.Field0074.showValue;
                if (!Filter.IsNullOrEmpty())
                {
                    if (rtn.DepartmentCode.IsNullOrEmpty()) continue;
                    if (Filter.Contains(rtn.DepartmentCode))
                    {
                        rtns.Results.Add(rtn);
                    }
                }
                else rtns.Results.Add(rtn);
            }

            rtns.Total = rtns.Results.Count;
            return rtns;
        }
    }
}
